use bevy::prelude::*;

use crate::{game_state::GameTimer, main_menu::button_color_system, GameState};
pub fn game_over_menu_plugin(app: &mut App) {
    app.add_systems(
        OnEnter(GameState::GameOver),
        game_over_menu_setup.in_set(GameOverMenuSet),
    )
    .add_systems(
        Update,
        (button_action, button_color_system).in_set(GameOverMenuSet),
    )
    .add_systems(OnExit(GameState::GameOver), exit_menu);
}

#[derive(Component)]
struct GameOverMenu;

#[derive(SystemSet, Debug, Clone, PartialEq, Eq, Hash)]
pub struct GameOverMenuSet;

#[derive(Component)]
enum ButtonAction {
    Retry,
    MainMenu,
}

fn exit_menu(
    to_despawn: Query<Entity, With<GameOverMenu>>,
    game_timer: Query<Entity, With<GameTimer>>,
    mut commands: Commands,
) {
    commands.entity(game_timer.single()).despawn();
    for e in &to_despawn {
        commands.entity(e).despawn_recursive();
    }
}

fn game_over_menu_setup(mut commands: Commands, timer: Query<&GameTimer>) {
    commands
        .spawn((
            NodeBundle {
                style: Style {
                    align_items: AlignItems::Center,
                    justify_content: JustifyContent::Center,

                    flex_direction: FlexDirection::Column,
                    width: Val::Percent(100.0),
                    height: Val::Percent(100.0),
                    ..default()
                },
                ..default()
            },
            GameOverMenu,
        ))
        .with_children(|parent| {
            parent.spawn(TextBundle::from_section(
                format!(
                    "Your score: {:.0}",
                    timer.single().game_time.elapsed_secs() * 100.
                ),
                TextStyle {
                    font_size: 96.,
                    ..default()
                },
            ));
            let btn_style = Style {
                width: Val::Px(300.0),
                height: Val::Px(80.0),
                margin: UiRect::all(Val::Px(20.0)),

                justify_content: JustifyContent::Center,
                align_items: AlignItems::Center,
                ..default()
            };
            let btn_txt_style = TextStyle {
                font_size: 64.,
                ..default()
            };
            parent
                .spawn((
                    ButtonBundle {
                        style: btn_style.clone(),
                        background_color: Color::rgb(0.3, 0.3, 0.3).into(),
                        ..default()
                    },
                    ButtonAction::Retry,
                ))
                .with_children(|parent| {
                    parent.spawn(TextBundle::from_section("Retry", btn_txt_style.clone()));
                });
            parent
                .spawn((
                    ButtonBundle {
                        style: btn_style.clone(),
                        background_color: Color::rgb(0.3, 0.3, 0.3).into(),
                        ..default()
                    },
                    ButtonAction::MainMenu,
                ))
                .with_children(|parent| {
                    parent.spawn(TextBundle::from_section("Main Menu", btn_txt_style.clone()));
                });
        });
}

fn button_action(
    interaction_query: Query<(&Interaction, &ButtonAction), (Changed<Interaction>, With<Button>)>,
    mut game_state: ResMut<NextState<GameState>>,
) {
    for (interaction, action) in &interaction_query {
        if *interaction == Interaction::Pressed {
            match action {
                ButtonAction::MainMenu => {
                    game_state.set(GameState::MainMenu);
                }
                ButtonAction::Retry => {
                    game_state.set(GameState::InGame);
                }
            }
        }
    }
}
