use bevy::{prelude::*, time::Stopwatch};

use crate::{GameState, GameplaySet};

#[derive(Component)]
pub struct GameTimer {
    pub game_time: Stopwatch,
}

#[derive(Component)]
struct ScoreText;

pub fn state_and_ui_plugin(app: &mut App) {
    app.add_systems(
        OnEnter(GameState::InGame),
        (setup_gamestate, setup_ui).in_set(GameplaySet),
    )
    .add_systems(Update, update_time.in_set(GameplaySet))
    .add_systems(OnExit(GameState::InGame), cleanup_ui.in_set(GameplaySet));
}

fn cleanup_ui(e: Query<Entity, With<ScoreText>>, mut commands: Commands) {
    commands.entity(e.single()).despawn()
}

fn setup_gamestate(mut commands: Commands) {
    commands.spawn(GameTimer {
        game_time: Stopwatch::new(),
    });
}

fn setup_ui(mut commands: Commands) {
    commands.spawn((
        TextBundle::from_sections([
            TextSection::new("Score: ", TextStyle::default()),
            TextSection::new("0", TextStyle::default()),
        ])
        .with_style(Style {
            position_type: PositionType::Absolute,
            top: Val::Px(5.0),
            right: Val::Px(5.0),
            ..default()
        }),
        ScoreText,
    ));
}

fn update_time(
    mut state_txt: Query<&mut Text, With<ScoreText>>,
    mut game_state: Query<&mut GameTimer>,
    time: Res<Time>,
) {
    let mut txt = state_txt.single_mut();
    let time = game_state
        .single_mut()
        .game_time
        .tick(time.delta())
        .elapsed_secs()
        * 100.;

    txt.sections[1].value = format!("{time:.0}");
}
